#include <stdio.h>    
#include <stdlib.h>    
#include <sys/types.h> 
#include <errno.h>
#include <string.h>
#include <libusb.h> 

#define BULK_EP_OUT1     0x82   
#define BULK_EP_OUT2     0x83
#define BULK_EP_IN       0x01

#define IMAGE_PACKET_SIZE (96 * 96 * 2)
char img_buf[IMAGE_PACKET_SIZE];

char cmd01[2] = { 0x40, 0x3f };
char cmd02[2] = { 0x00, 0x09 };

int write_to_file(char* buf, int size) {
    FILE *fp;
    fp = fopen("./out.fp","w");
    if (fp < 0) return 1;
    fwrite(buf, 1, size, fp);
    fclose(fp);
    return 0;
}

int main() {
    int r0, i, devs_count = 0, libusb_config = 0, transferred = 0;
    
    struct libusb_device **devs;
    struct libusb_device *dev;
    struct libusb_device_descriptor desc;
    struct libusb_device_handle *handle = NULL;
    
    r0 = libusb_init(NULL);
    if (r0 < 0) return 1;
    
    devs_count = libusb_get_device_list(NULL, &devs);
    if (devs_count < 0) {
        libusb_exit(NULL);
        printf("Error %d\n", r0);
        return 2;
    }
    
    while (1) {
        dev = devs[i];
        if (dev != NULL) {
            r0 = libusb_get_device_descriptor(dev, &desc);
            if (r0 < 0) continue;

            if ((desc.idVendor == 0x04f3) && (desc.idProduct == 0x0c03)) {
                r0 = 0;
                printf("Device with vid %x pid %x found.\n", desc.idVendor, desc.idProduct);
                break;
            }
        } else {
            r0 = 3;
            break;
        }
        
        i++;
        if (i > devs_count) {
            r0 = 99;
            goto app_exit;
        }
    }
    
    if (r0 != 0) goto app_exit;
    
    /*****************************************************/
    
    r0 = libusb_open(dev, &handle);
    if (r0 < 0) {
        r0 = 5;
        goto app_exit;
    }
    
    r0 = libusb_get_configuration(handle, &libusb_config);
    if (r0 != 0) {
        r0 = 98;
        goto app_exit_2;
    }
    
    printf("Config number is %d\n", libusb_config);
    
    if (libusb_config > 1) {
        r0 = libusb_set_configuration(handle, 1);
        if (r0 != 0) {
            r0 = 97;
            goto app_exit_2;
        }
        printf("Config set to 1\n");
    }
    
    if (libusb_kernel_driver_active(handle, 0) == 1) {
        if(libusb_detach_kernel_driver(handle, 0) != 0) {
            r0 = 96;
            goto app_exit_2;
        }
    }
    
    r0 = libusb_claim_interface(handle, 0);
    if (r0 != 0) {
        r0 = 95;
        goto app_exit_2;
    }
    
    r0 = libusb_bulk_transfer(handle, BULK_EP_IN, cmd01, 2, &transferred, 0);
    if((r0 == 0) && (transferred == 2)) {
        printf("CMD1 sent\n");
    }
    
    r0 = libusb_bulk_transfer(handle, BULK_EP_OUT2, img_buf, 1, &transferred, 0); // wait for press
    printf("Received byte %d\n", img_buf[0]);
    
    r0 = libusb_bulk_transfer(handle, BULK_EP_IN, cmd02, 2, &transferred, 0);
    if((r0 == 0) && (transferred == 2)) {
        printf("CMD2 sent\n");
    }
    
    r0 = libusb_bulk_transfer(handle, BULK_EP_OUT1, img_buf, IMAGE_PACKET_SIZE, &transferred, 0);
    printf("Received %d\n", transferred);
    write_to_file(img_buf, IMAGE_PACKET_SIZE);
    
    system("convert -depth 16 -size 96x96+0 gray:out.fp ./out.png");
    system("rm ./out.fp");
    
app_exit_2:
    libusb_close(handle); 
    
    /*****************************************************/
    
app_exit:
    libusb_free_device_list(devs, 1);
    libusb_exit(NULL);
    
    if (r0 != 0) {
        printf("Error %d\n", r0);
    }
    
    return r0;
}
